"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var mysql = require("mysql");
var utils_1 = require("../commons/utils/utils");
var BaseHandler_1 = require("./BaseHandler");
/**
 * MySql Handler.
 *
 * @export
 * @class MysqlHandler
 * @extends {BaseHandler}
 */
var MysqlHandler = (function (_super) {
    tslib_1.__extends(MysqlHandler, _super);
    /**
     * Constructor for the MySqlHandler.
     *
     * @param {mysq.IConnectionConfig} options Connection parameters.
     */
    function MysqlHandler(options) {
        var _this = _super.call(this, 'mysql') || this;
        _this.options = options;
        _this.connection = mysql.createConnection({
            host: _this.options.host,
            port: _this.options.port,
            user: _this.options.user,
            password: _this.options.password,
            database: 'information_schema',
        });
        return _this;
    }
    /**
     * Connect to MySql based on the connection data.
     */
    MysqlHandler.prototype.connect = function () {
        this.connection.connect();
    };
    /**
     * Reads the database schema, processes it and returns a normalized version of it.
     *
     * @returns {Promise<Schema>} database schema
     */
    MysqlHandler.prototype.readSchema = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        _this.getTables().then(function (tablesNames) {
                            var tables = tablesNames.map(function (tableName) { return tslib_1.__awaiter(_this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
                                return [2 /*return*/, this.getTableSchema(tableName)];
                            }); }); });
                            Promise.all(tables).then(function (schema) {
                                resolve(_this.normalizeRelations(schema));
                            }).catch(reject);
                        }).catch(reject);
                    })];
            });
        });
    };
    /**
     * Reads the information schema and returns an array of tables.
     *
     * @returns {Promise<string[]>} array of table names.
     */
    MysqlHandler.prototype.getTables = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        _this.connection.query("SELECT TABLE_NAME FROM TABLES WHERE TABLE_SCHEMA = '" + _this.options.database + "';", function (err, results) {
                            /* istanbul ignore next */
                            if (err) {
                                return reject(err);
                            }
                            var tables = results.map(function (result) { return result.TABLE_NAME; });
                            return resolve(tables);
                        });
                    })];
            });
        });
    };
    /**
     * Reads the schema for a given table.
     *
     * @param {string} tableName table name
     * @returns {Promise<Table>} table schema
     */
    MysqlHandler.prototype.getTableSchema = function (tableName) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        var table = {
                            name: tableName,
                            columns: [],
                        };
                        _this.connection.query("SELECT\n          COLUMN_NAME,\n          IS_NULLABLE,\n          DATA_TYPE,\n          CHARACTER_MAXIMUM_LENGTH,\n          COLUMN_KEY,\n          COLUMN_TYPE\n        FROM COLUMNS WHERE TABLE_SCHEMA = '" + _this.options.database + "' AND TABLE_NAME = '" + tableName + "';", function (err, columns) {
                            /* istanbul ignore next */
                            if (err) {
                                return reject(err);
                            }
                            _this.getRelationsForTable(tableName).then(function (relations) {
                                columns.forEach(function (result) {
                                    var column = _this.normalizeColumnSchema(result);
                                    var relation = relations.filter(function (rel) { return rel.name === column.name; }).pop();
                                    if (relation) {
                                        column.foreignKey = true;
                                        column.dataType.references = relation;
                                        column.dataType.type = utils_1.default.toTitleCase(relation.table);
                                        column.name = utils_1.default.singular(relation.table);
                                    }
                                    table.columns.push(column);
                                });
                                return resolve(table);
                            }).catch(reject);
                        });
                    })];
            });
        });
    };
    /**
     * Reads all the relations for a given table.
     *
     * @param {string} table table name
     * @return {Promise<TableReference[]>} foreign key relations
     */
    MysqlHandler.prototype.getRelationsForTable = function (table) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        _this.connection.query("SELECT\n          COLUMN_NAME,\n          rc.REFERENCED_TABLE_NAME,\n          REFERENCED_COLUMN_NAME\n        FROM REFERENTIAL_CONSTRAINTS rc JOIN\n          KEY_COLUMN_USAGE cu ON cu.CONSTRAINT_NAME = rc.CONSTRAINT_NAME\n        WHERE rc.CONSTRAINT_SCHEMA = '" + _this.options.database + "' AND rc.TABLE_NAME = '" + table + "'", function (err, relations) {
                            /* istanbul ignore next */
                            if (err) {
                                return reject(err);
                            }
                            var references = relations.map(function (relation) {
                                var reference = {
                                    name: relation.COLUMN_NAME,
                                    table: relation.REFERENCED_TABLE_NAME,
                                    column: relation.REFERENCED_COLUMN_NAME,
                                };
                                return reference;
                            });
                            resolve(references);
                        });
                    })];
            });
        });
    };
    /**
     * Closes the MySql connection.
     */
    MysqlHandler.prototype.close = function () {
        this.connection.end();
    };
    return MysqlHandler;
}(BaseHandler_1.default));
exports.default = MysqlHandler;
