import { Schema, Table, Column } from '../types';
/**
 * Class which holds helper tools when working with schema.
 *
 * @export
 * @class SchemaUtil
 */
export declare class SchemaUtil {
    /**
     * Get normalized schema by type
     * @param {Schema} schema - collection of tables
     * @param {string} type - data type normalized (E.g: User, Account)
     * @return {Table} table from schema that matches type.
     */
    getNormalizedTableByType(schema: Schema, type: string): Table | undefined;
    /**
     * Convert values from COLUMN_TYPE column
     * @param {Column} column - schema.
     * @return {Column} converted column
     */
    convertValues(column: Column): Column;
    /**
     * Convert array of values to raml values
     * @param {string[]} values - ['Yes', 'No']
     * @return {string} - 'Yes | No'
     */
    valuesToRamlDataType(values: string[]): string;
    /**
     * Checks if a circular reference exists between the source and target table.
     *
     * @param {Table} source - source table
     * @param {Column} column - column which references another table
     * @param {Schema} schema - whole schema
     * @return {boolean} true if a circular reference exists in the target table
     */
    isCircularRelation(source: Table, column: Column, schema: Schema): boolean;
    /**
     * Checks if a circular reference is required.
     *
     * @param {Table} source - source table
     * @param {Column} column - column which references another table
     * @param {Schema} schema - whole schema
     * @return {boolean} true if a circular reference is required
     */
    circularRelationIsRequired(source: Table, column: Column, schema: Schema): boolean;
    /**
     * Checks if a circular reference is array.
     *
     * @param {Table} source - source table
     * @param {Column} column - column which references another table
     * @param {Schema} schema - whole schema
     * @return {boolean} true if a circular reference is array
     */
    circularRelationIsArray(source: Table, column: Column, schema: Schema): boolean;
    /**
     * Get related table names for a given table.
     *
     * @param {Table} table - given table
     * @returns {string[]} - related tables
     */
    getRelatedTablesForTable(table: Table): string[];
}
declare const _default: SchemaUtil;
export default _default;
