"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var chalk = require("chalk");
var child_process_1 = require("child_process");
var fileUtil_1 = require("../commons/utils/fileUtil");
var directoryStructure_1 = require("../commons/constants/directoryStructure");
var packageJson_1 = require("../generators/api/packageJson");
var ormFileOperations_1 = require("./api/ormFileOperations");
var configurationsFileOperations_1 = require("./api/configurationsFileOperations");
var modelsFileOperations_1 = require("./api/modelsFileOperations");
var repositoriesFileOperations_1 = require("./api/repositoriesFileOperations");
var servicesFileOperations_1 = require("./api/servicesFileOperations");
var controllersFileOperations_1 = require("./api/controllersFileOperations");
var commonsFileOperations_1 = require("./api/commonsFileOperations");
/**
 * Class which implements the logic for implementing api generation file related actions.
 *
 * @export
 * @class ApiFileOperations
 */
var ApiFileOperations = (function () {
    function ApiFileOperations(filePath) {
        this.filePath = fileUtil_1.default.normalizePath(filePath);
    }
    /**
     * Create directory structure for the application
     * @returns {Promise<boolean[]>} created directory structure.
     */
    ApiFileOperations.prototype.createDirectoryStructure = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var promises;
            return tslib_1.__generator(this, function (_a) {
                promises = [Promise.resolve(true)];
                /* istanbul ignore next */
                if (!fileUtil_1.default.isDirectory(this.filePath)) {
                    return [2 /*return*/, Promise.reject('Invalid directory path')];
                }
                Object.keys(directoryStructure_1.default.API_STRUCTURE).map(function (directory) {
                    promises.push(fileUtil_1.default.createDirectory(fileUtil_1.default.joinPaths(_this.filePath, directoryStructure_1.default.API_STRUCTURE[directory])));
                });
                return [2 /*return*/, Promise.all(promises)];
            });
        });
    };
    /**
     * Creates package json
     *
     * @param {PackageJsonInfo} options - package json options
     * @param {string} dialect - database dialect
     * @returns {Promise<boolean>} created package json
     */
    ApiFileOperations.prototype.createPackageJson = function (options, dialect) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                console.log("running: " + chalk.green('initialize package.json'));
                return [2 /*return*/, fileUtil_1.default.writeFile(fileUtil_1.default.joinPaths(this.filePath, 'package.json'), packageJson_1.default.getPackageJsonAsString(options, dialect))];
            });
        });
    };
    /**
     * Install dependencies for api stub
     *
     * @param {string} dialect database type
     * @return {Promise<boolean>} installed dependencies
     */
    ApiFileOperations.prototype.installDependencies = function (dialect) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        console.log("running: " + chalk.green("npm i --save express body-parser morgan " + _this.getDependenciesForDb(dialect)));
                        child_process_1.exec("cd " + _this.filePath + " && npm i --save express body-parser morgan " + _this.getDependenciesForDb(dialect), function (errDependencies) {
                            /* istanbul ignore next */
                            if (errDependencies) {
                                return reject(errDependencies);
                            }
                            console.log("running: " + chalk.green('npm i --save-dev nodemon'));
                            child_process_1.exec("cd " + _this.filePath + " && npm i --save-dev nodemon", function (errDev) {
                                /* istanbul ignore next */
                                if (errDev) {
                                    return reject(errDev);
                                }
                                resolve(true);
                            });
                        });
                    })];
            });
        });
    };
    /**
     * Initialize ODM
     *
     * @param {string} dialect database dialect
     * @returns {Promise<boolean>} initialized orm
     */
    ApiFileOperations.prototype.initializeORM = function (dialect) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (dialect) {
                    case 'MySql': {
                        return [2 /*return*/, ormFileOperations_1.default.initSequelize(this.filePath)];
                    }
                    default:
                        return [2 /*return*/, Promise.reject('Dialect not supported')];
                }
                return [2 /*return*/];
            });
        });
    };
    /**
     * Generate config
     * @param {ConnectionData} connection - connection info
     * @param {Schema} schema - database schema tables
     * @returns {Promise<boolean[]>} - true if generated config
     * @memberOf ApiFileOperations
     */
    ApiFileOperations.prototype.initializeConfig = function (connection, schema) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                console.log("running: " + chalk.green('initialize src/config files'));
                return [2 /*return*/, Promise.all([
                        configurationsFileOperations_1.default.initializeConfig(this.filePath),
                        configurationsFileOperations_1.default.initializeGetEnvBasedConfig(this.filePath),
                        configurationsFileOperations_1.default.initializeDbConfig(connection, this.filePath),
                        configurationsFileOperations_1.default.initializeCorsConfig(this.filePath),
                        configurationsFileOperations_1.default.initializeExpressConfig(this.filePath),
                        configurationsFileOperations_1.default.initializeRouterConfig(this.filePath, schema),
                        configurationsFileOperations_1.default.initializeIndex(this.filePath),
                    ])];
            });
        });
    };
    /**
     * Initialize commons
     * @return {Promise<boolean[]>}
     */
    ApiFileOperations.prototype.initializeCommons = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                console.log("running: " + chalk.green('initialize src/commons'));
                return [2 /*return*/, Promise.all([
                        commonsFileOperations_1.default.initializeUtil(this.filePath),
                        commonsFileOperations_1.default.initializeConstants(this.filePath),
                    ])];
            });
        });
    };
    /**
     * Initialize ODM
     *
     * @param {string} dialect database dialect
     * @param {Schema} schema source schema for api generation
     * @returns {Promise<boolean[]>} initialized odm
     */
    ApiFileOperations.prototype.initializeModels = function (dialect, schema) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                console.log("running: " + chalk.green("initialize src/models for " + dialect));
                switch (dialect) {
                    case 'MySql': {
                        return [2 /*return*/, modelsFileOperations_1.default.initializeSequelizeModels(this.filePath, schema)];
                    }
                    default:
                        return [2 /*return*/, Promise.reject('Dialect not supported')];
                }
                return [2 /*return*/];
            });
        });
    };
    /**
     * Initialize Repositories
     *
     * @param {string} dialect database dialect
     * @param {Schema} schema source schema for api generation
     * @returns {Promise<boolean[]>} initialized repositories
     */
    ApiFileOperations.prototype.initializeRepositories = function (dialect, schema) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                console.log("running: " + chalk.green("initialize src/repositories for " + dialect));
                switch (dialect) {
                    case 'MySql': {
                        return [2 /*return*/, repositoriesFileOperations_1.default.initializeSequelizeRepositories(this.filePath, schema)];
                    }
                    default:
                        return [2 /*return*/, Promise.reject('Dialect not supported')];
                }
                return [2 /*return*/];
            });
        });
    };
    /**
     * Initialize Services
     * @param schema
     * @return {Promise<Promise<boolean[]>>}
     */
    ApiFileOperations.prototype.initializeServices = function (schema) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                console.log("running: " + chalk.green('initialize src/services'));
                return [2 /*return*/, servicesFileOperations_1.default.initializeServices(this.filePath, schema)];
            });
        });
    };
    /**
     * Initialize Controllers
     * @param schema
     * @return {Promise<boolean[]>}
     */
    ApiFileOperations.prototype.initializeControllers = function (schema) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                console.log("running: " + chalk.green('initialize src/controllers'));
                return [2 /*return*/, controllersFileOperations_1.default.initializeControllers(this.filePath, schema)];
            });
        });
    };
    /**
     * Get required dependencies based on database type
     *
     * @param {string} databaseType database type
     * @return {string} required dependencies for the database type
     */
    ApiFileOperations.prototype.getDependenciesForDb = function (dialect) {
        switch (dialect) {
            case 'MySql': {
                return 'sequelize sequelize-cli mysql2';
            }
            default: {
                return '';
            }
        }
    };
    /**
     * Returns file path.
     *
     * @returns {string} file path
     */
    ApiFileOperations.prototype.getFilePath = function () {
        return this.filePath;
    };
    return ApiFileOperations;
}());
exports.default = ApiFileOperations;
