"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var fileUtil_1 = require("../commons/utils/fileUtil");
var directoryStructure_1 = require("../commons/constants/directoryStructure");
var types_1 = require("../generators/spec/types");
var examples_1 = require("../generators/spec/examples");
var spec_1 = require("../generators/spec/spec");
var utils_1 = require("../commons/utils/utils");
var cacheUtil_1 = require("../commons/utils/cacheUtil");
var config_1 = require("../configs/config");
/**
 * Class which implements the logic for implementing raml generation file related actions.
 *
 * @export
 * @class RamlFileOperations
 */
var RamlFileOperations = (function () {
    function RamlFileOperations(filePath) {
        this.filePath = fileUtil_1.default.normalizePath(filePath);
    }
    /**
     * Create directory structure for the application
     * @returns {Promise<boolean[]>} created directory structure.
     */
    RamlFileOperations.prototype.createDirectoryStructure = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var promises;
            return tslib_1.__generator(this, function (_a) {
                promises = [Promise.resolve(true)];
                /* istanbul ignore next */
                if (!fileUtil_1.default.isDirectory(this.filePath)) {
                    return [2 /*return*/, Promise.reject('Invalid directory path')];
                }
                Object.keys(directoryStructure_1.default.RAML_STRUCTURE).map(function (directory) {
                    promises.push(fileUtil_1.default.createDirectory(fileUtil_1.default.joinPaths(_this.filePath, directoryStructure_1.default.RAML_STRUCTURE[directory])));
                });
                return [2 /*return*/, Promise.all(promises)];
            });
        });
    };
    /**
     * Create .raml type file for every table inside schema
     * @param {Schema} schema - schema tables(list of tables)
     * @returns {Promise.<boolean[]>} generated type files.
     */
    RamlFileOperations.prototype.generateSchemaTypeFiles = function (schema) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var promises;
            return tslib_1.__generator(this, function (_a) {
                promises = [Promise.resolve(true)];
                schema.map(function (table) {
                    promises.push(fileUtil_1.default.writeFile(fileUtil_1.default.joinPaths(_this.filePath, directoryStructure_1.default.RAML_STRUCTURE.TYPES, utils_1.default.toTitleCase(table.name) + ".raml"), types_1.default.generateTypeContent(table)));
                });
                return [2 /*return*/, Promise.all(promises)];
            });
        });
    };
    /**
     * Generate api.raml spec for the whole api.
     *
     * @param {Schema} schema - database schema
     * @param {RAMLApplicationInfo} options - application info
     * @returns {Promise<boolean>} - true if spec generated
     */
    RamlFileOperations.prototype.generateSchemaApiFiles = function (schema, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, fileUtil_1.default.writeFile(fileUtil_1.default.joinPaths(this.filePath, 'api.raml'), spec_1.default.generateContent(schema, options))];
            });
        });
    };
    /**
     * Generate .json entity for every table from schema.
     * File will be saved on path: ./raml/types/entityName.json
     * @param {Schema} schema - schema containing tables.
     * @return {Promise.<boolean[]>} generated examle files.
     */
    RamlFileOperations.prototype.generateSchemaExampleFiles = function (schema) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var promises;
            return tslib_1.__generator(this, function (_a) {
                promises = [Promise.resolve(true)];
                schema.map(function (table) {
                    var typeExampleGenerated = examples_1.default.generateTypeExampleContent(schema, table, config_1.default.INITIAL_DEPTH_LEVEL);
                    promises.push(fileUtil_1.default.writeFile(fileUtil_1.default.joinPaths(_this.filePath, directoryStructure_1.default.RAML_STRUCTURE.EXAMPLES, typeExampleGenerated.type + ".json"), utils_1.default.convertToJSON(typeExampleGenerated.data)));
                });
                return [2 /*return*/, Promise.all(promises)];
            });
        });
    };
    /**
     * Generate .json Array entity for every array of objects saved in cache
     * @return {Promise.<boolean[]>} generated example files.
     */
    RamlFileOperations.prototype.generateSchemaExamplesFilesFromCache = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            var promises;
            return tslib_1.__generator(this, function (_a) {
                promises = [Promise.resolve(true)];
                Object.keys(cacheUtil_1.default.getByPrimeKey(examples_1.default.PRIME_KEY))
                    .filter(function (key) { return key.includes('[]'); })
                    .map(function (key) {
                    promises.push(fileUtil_1.default.writeFile(fileUtil_1.default.joinPaths(_this.filePath, directoryStructure_1.default.RAML_STRUCTURE.EXAMPLES, utils_1.default.pluraliseWordArray(key) + ".json"), utils_1.default.convertToJSON(cacheUtil_1.default.get(examples_1.default.PRIME_KEY, key))));
                });
                return [2 /*return*/, Promise.all(promises)];
            });
        });
    };
    /**
     * Returns file path.
     *
     * @returns {string} file path
     */
    RamlFileOperations.prototype.getFilePath = function () {
        return this.filePath;
    };
    return RamlFileOperations;
}());
exports.default = RamlFileOperations;
