"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../../../commons/utils/utils");
var typeUtil_1 = require("../../../commons/utils/typeUtil");
/**
 * Class which implements the logic for generating valid sequelize files.
 *
 * @export
 * @class SequelizeModelGenerator
 */
var SequelizeModelGenerator = (function () {
    function SequelizeModelGenerator() {
    }
    /**
     * Returns the string version of the code which will manage the sequelzie models.
     *
     * @returns {string} - sequelize models registry
     * @memberof SequelizeModelGenerator
     */
    SequelizeModelGenerator.prototype.getModelsRegistry = function () {
        return "'use strict';\nconst fs        = require('fs');\nconst path      = require('path');\nconst Sequelize = require('sequelize');\nconst basename  = path.basename(module.filename);\nconst env       = process.env.NODE_ENV || 'development';\nconst config    = require(__dirname + '/../configs/database')[env];\nconst db        = {};\nlet sequelize;\n\nif (config.use_env_variable) {\n  sequelize = new Sequelize(process.env[config.use_env_variable]);\n} else {\n  sequelize = new Sequelize(config.database, config.username, config.password, config);\n}\nfs\n  .readdirSync(__dirname)\n  .filter(function(file) {\n    return (file.indexOf('.') !== 0) && (file !== basename) && (file.slice(-3) === '.js');\n  })\n  .forEach(function(file) {\n    var model = sequelize['import'](path.join(__dirname, file));\n    db[model.name] = model;\n  });\nObject.keys(db).forEach(function(modelName) {\n  if (db[modelName].associate) {\n    db[modelName].associate(db);\n  }\n});\ndb.sequelize = sequelize;\ndb.Sequelize = Sequelize;\nmodule.exports = db;\n";
    };
    /**
     * Returns the sequelize config
     *
     * @param {ConnectionData} connection - connection data
     * @returns {string} - connection string
     * @memberof SequelizeModelGenerator
     */
    SequelizeModelGenerator.prototype.getConfig = function (connection) {
        var config = {
            development: {
                uri: connection.dialect.toLowerCase() + "://" + connection.user + ":" + connection.password + "@" + connection.host + ":" + connection.port + "/" + connection.database,
                username: connection.user,
                password: connection.password,
                database: connection.database,
                host: connection.host,
                dialect: connection.dialect.toLowerCase(),
                port: connection.port,
            },
            production: {
                uri: 'mysql://user:password@host:port/database',
                username: 'user',
                password: 'password',
                database: 'database',
                host: 'host',
                dialect: 'mysql',
                port: 'port',
            },
        };
        return "module.exports = " + JSON.stringify(config, undefined, 2);
    };
    /**
     * Generate models for a given schema.
     *
     * @param {Schema} schema - api schema
     * @return {{name: string, content: string}[]}
     */
    SequelizeModelGenerator.prototype.getModels = function (schema) {
        var _this = this;
        var models = [];
        schema.forEach(function (table) {
            models.push({
                name: utils_1.default.toTitleCase(table.name) + ".js",
                content: _this.getModelForTable(table),
            });
        });
        return models;
    };
    /**
     * Generate contents for each model based on the table schema.
     *
     * @param {Table} table - source
     * @return {string} content of sequelize model as string.
     */
    SequelizeModelGenerator.prototype.getModelForTable = function (table) {
        return "module.exports = (sequelize, DataTypes) => {\n  const " + utils_1.default.toTitleCase(table.name) + " = sequelize.define('" + utils_1.default.singular(table.name).toLowerCase() + "', {\n" + this.getBasicDataTypes(table) + "\n  }, {\n    tableName: '" + table.name + "',\n    timestamps: false,\n    underscored: true,\n  });\n  " + utils_1.default.toTitleCase(table.name) + ".associate = (models) => {\n" + this.getRelations(table) + "\n  }\n  return " + utils_1.default.toTitleCase(table.name) + ";\n};";
    };
    /**
     * Generate contents for the basic data types.
     * @param {Table} table - source
     * @return {string} generated content.
     */
    SequelizeModelGenerator.prototype.getBasicDataTypes = function (table) {
        var _this = this;
        var dataTypes = '';
        table.columns.forEach(function (column) {
            if (!typeUtil_1.default.isDefaultType(column.dataType.type)) {
                return;
            }
            dataTypes += "    " + column.name + ": {\n      type: " + _this.getType(column.dataType) + ",\n      allowNull: " + (column.allowNull ? 'true' : 'false') + ",\n      unique: " + (column.unique ? 'true' : 'false') + ",\n      primaryKey: " + (column.primary ? 'true' : 'false') + ",\n      " + (column.primary && column.dataType.type === 'number' ? 'autoIncrement: true,' : '') + "\n    },\n";
        });
        return dataTypes;
    };
    /**
     * Returns the string version of the sequelize type.
     * @param {DataType} type source
     * @return {string} sequelize type
     */
    SequelizeModelGenerator.prototype.getType = function (type) {
        switch (type.type) {
            case 'string':
                return 'DataTypes.STRING()';
            case 'number':
                return 'DataTypes.INTEGER()';
            case 'enum':
                return "DataTypes.ENUM(" + typeUtil_1.default.getEnumValuesAsString(type) + ")";
            /* istanbul ignore next */
            default:
                return '';
        }
    };
    /**
     * Generate association methods for types.
     * @param {Table} table source
     * @return {string} - generated content
     */
    SequelizeModelGenerator.prototype.getRelations = function (table) {
        var modelName = utils_1.default.toTitleCase(table.name);
        return table.columns.filter(function (column) { return !typeUtil_1.default.isDefaultType(column.dataType.type); }).map(function (column) {
            switch (column.dataType.relationType) {
                case '1-1': {
                    if (column.dataType.isRelationHolder) {
                        return "    " + modelName + ".hasOne(models." + column.dataType.type.toLowerCase() + ");";
                    }
                    else {
                        return "    " + modelName + ".belongsTo(models." + column.dataType.type.toLowerCase() + ");";
                    }
                }
                case '1-n': {
                    if (column.dataType.isRelationHolder) {
                        return "    " + modelName + ".hasMany(models." + column.dataType.type.toLowerCase() + ");";
                    }
                    else {
                        return "    " + modelName + ".belongsTo(models." + column.dataType.type.toLowerCase() + ");";
                    }
                }
                case 'n-n': {
                    if (modelName.localeCompare(column.dataType.type) > 0) {
                        return "    " + modelName + ".belongsToMany(models." + column.dataType.type.toLowerCase() + ", {through: '" + utils_1.default.pluralize(modelName).toLowerCase() + "_" + utils_1.default.pluralize(column.dataType.type).toLowerCase() + "'});";
                    }
                    else {
                        return "    " + modelName + ".belongsToMany(models." + column.dataType.type.toLowerCase() + ", {through: '" + utils_1.default.pluralize(column.dataType.type).toLowerCase() + "_" + utils_1.default.pluralize(modelName).toLowerCase() + "'});";
                    }
                }
                default:
                    return '';
            }
        }).join('\n');
    };
    return SequelizeModelGenerator;
}());
exports.default = SequelizeModelGenerator;
