import { Schema, Column, MySqlColumnSchema } from '../commons/types';
/**
 * MySql schema pre processor.
 *
 * @export
 * @class MysqlSchemaPreprocessor
 */
export default class MysqlSchemaPreprocessor {
    /**
     * Normalize the table schema.
     *
     * @param {*} columnSchema column schema to be normalized
     * @returns {{}} normalized column schema
     */
    convertToStandardSchema(columnSchema: MySqlColumnSchema): Column;
    /**
     * Normalizes all foreign key relations, by creating the respective columns in the target tables.
     *
     * @param {*} schema db schema
     * @returns {*} normalized db schema
     */
    normalizeSchemaRelations(schema: Schema): Schema;
    /**
     * Normalizes one to one relations.
     * Parses the table list and checks for tables
     * which have unique foreign keys, are not CrossReferenceTables(many to many)
     * and adds the foreign column on both sides of the relation.
     *
     * @param {*} schema db schema
     * @returns {*} normalized db schema
     */
    private normalizeOneToOneRelations(schema);
    /**
     * Normalizes one to many relations.
     * Parses the table list and checks for tables
     * which have foreign keys, are not CrossReferenceTables(many to many)
     * and adds the foreign column on the holder.
     *
     * @param {*} schema db schema
     * @returns {*} normalized db schema
     */
    private normalizeOneToManyRelations(schema);
    /**
     * Normalizes many to many relations.
     * Parses the table list and checks for tables
     * which are CrossReferenceTables(many to many)
     * and adds the foreign columns to the source and target tables.
     *
     * @param {*} schema db schema
     * @returns {*} normalized db schema
     */
    private normalizeManyToManyRelations(schema);
    /**
     * Strips tables with no columns from schema.
     *
     * @param {*} schema db schema
     * @returns {Array<*>} cleaned up schema
     */
    private stripEmptyTables(schema);
    /**
     * Checks if the table has any foreign key columns.
     *
     * @param {*} table db table
     * @returns {boolean} checked if it contains foreign keys.
     */
    private tableHasForeignKeys(table);
    /**
     * Checks if the table is a cross reference map(association table for many to many relations).
     *
     * @param {*} table table schema
     * @returns {boolean} checked if cross reference table.
     */
    private tableIsCrossReferenceTable(table);
    /**
     * Adds a column to a table in the schema.
     *
     * @param {*} schema db schema
     * @param {string} tableName table in which to add column
     * @param {*} column column to be added
     * @returns {Array|*} updated schema
     */
    private addColumnToTable(schema, tableName, column);
    /**
     * Removes a column from a table in the schema.
     *
     * @param {*} schema db schema
     * @param {string} tableName table from which we need to remove
     * @param {string} columnName column to be removed
     * @returns {Array|*} schema with removed column
     */
    private removeColumnFromTable(schema, tableName, columnName);
    /**
     * Remove unused properties from schema.
     *
     * @param {*} schema db schema
     * @returns {Array|*} cleaned up db schema
     */
    private cleanupUnusedPropertiesFromColumns(schema);
}
