"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var typeUtil_1 = require("../commons/utils/typeUtil");
var utils_1 = require("../commons/utils/utils");
var schemaUtil_1 = require("../commons/utils/schemaUtil");
var logger_1 = require("../commons/logger");
var config_1 = require("../configs/config");
var CROSS_REFERENCE_COL_LENGTH = 2;
var FIRST_INDEX = 0;
var SECOND_INDEX = 1;
/**
 * MySql schema pre processor.
 *
 * @export
 * @class MysqlSchemaPreprocessor
 */
var MysqlSchemaPreprocessor = (function () {
    function MysqlSchemaPreprocessor() {
    }
    /**
     * Normalize the table schema.
     *
     * @param {*} columnSchema column schema to be normalized
     * @returns {{}} normalized column schema
     */
    MysqlSchemaPreprocessor.prototype.convertToStandardSchema = function (columnSchema) {
        var column = {
            name: columnSchema.COLUMN_NAME ? columnSchema.COLUMN_NAME : '',
            primary: Boolean(columnSchema.COLUMN_KEY && columnSchema.COLUMN_KEY === 'PRI'),
            unique: Boolean(columnSchema.COLUMN_KEY && (columnSchema.COLUMN_KEY === 'PRI' || columnSchema.COLUMN_KEY === 'UNI')),
            foreignKey: Boolean(columnSchema.COLUMN_KEY && columnSchema.COLUMN_KEY === 'MUL'),
            allowNull: Boolean(columnSchema.IS_NULLABLE && columnSchema.IS_NULLABLE === 'YES'),
            dataType: {
                type: typeUtil_1.default.convertSqlType(columnSchema.DATA_TYPE),
                size: columnSchema.CHARACTER_MAXIMUM_LENGTH ?
                    parseInt(columnSchema.CHARACTER_MAXIMUM_LENGTH, config_1.default.NUMERIC_BASE) :
                    undefined,
                rawValues: columnSchema.COLUMN_TYPE,
            },
        };
        return schemaUtil_1.default.convertValues(column);
    };
    /**
     * Normalizes all foreign key relations, by creating the respective columns in the target tables.
     *
     * @param {*} schema db schema
     * @returns {*} normalized db schema
     */
    MysqlSchemaPreprocessor.prototype.normalizeSchemaRelations = function (schema) {
        var normalizedSchema = schema;
        normalizedSchema = this.normalizeOneToOneRelations(normalizedSchema);
        normalizedSchema = this.normalizeOneToManyRelations(normalizedSchema);
        normalizedSchema = this.normalizeManyToManyRelations(normalizedSchema);
        normalizedSchema = this.cleanupUnusedPropertiesFromColumns(normalizedSchema);
        normalizedSchema = this.stripEmptyTables(normalizedSchema);
        logger_1.default.info(JSON.stringify(normalizedSchema));
        return normalizedSchema;
    };
    /**
     * Normalizes one to one relations.
     * Parses the table list and checks for tables
     * which have unique foreign keys, are not CrossReferenceTables(many to many)
     * and adds the foreign column on both sides of the relation.
     *
     * @param {*} schema db schema
     * @returns {*} normalized db schema
     */
    MysqlSchemaPreprocessor.prototype.normalizeOneToOneRelations = function (schema) {
        var _this = this;
        var updatedSchema = schema;
        schema.forEach(function (table) {
            if (!_this.tableHasForeignKeys(table) || _this.tableIsCrossReferenceTable(table)) {
                return table;
            }
            table.columns.forEach(function (column) {
                if (column.foreignKey && column.unique) {
                    var targetColumn = {
                        name: column.name,
                        primary: column.primary,
                        unique: true,
                        allowNull: false,
                        dataType: {
                            type: column.dataType.type,
                            isArray: false,
                            relationType: '1-1',
                        },
                    };
                    var sourceColumn = {
                        name: utils_1.default.singular(table.name).toLowerCase(),
                        primary: column.primary,
                        unique: true,
                        allowNull: true,
                        dataType: {
                            type: utils_1.default.toTitleCase(table.name),
                            isArray: false,
                            relationType: '1-1',
                            isRelationHolder: true,
                        },
                    };
                    updatedSchema = _this.addColumnToTable(updatedSchema, column.dataType.references ? column.dataType.references.table : '', sourceColumn);
                    updatedSchema = _this.addColumnToTable(updatedSchema, table.name, targetColumn);
                }
            });
            return table;
        });
        return updatedSchema;
    };
    /**
     * Normalizes one to many relations.
     * Parses the table list and checks for tables
     * which have foreign keys, are not CrossReferenceTables(many to many)
     * and adds the foreign column on the holder.
     *
     * @param {*} schema db schema
     * @returns {*} normalized db schema
     */
    MysqlSchemaPreprocessor.prototype.normalizeOneToManyRelations = function (schema) {
        var _this = this;
        var updatedSchema = schema;
        schema.forEach(function (table) {
            if (!_this.tableHasForeignKeys(table) || _this.tableIsCrossReferenceTable(table)) {
                return table;
            }
            table.columns.forEach(function (column) {
                if (column.foreignKey) {
                    var sourceColumn = {
                        name: column.name,
                        primary: column.primary,
                        unique: false,
                        allowNull: false,
                        dataType: {
                            type: utils_1.default.toTitleCase(column.name),
                            isArray: false,
                            relationType: '1-n',
                        },
                    };
                    var targetColumn = {
                        name: table.name,
                        primary: column.primary,
                        unique: false,
                        allowNull: true,
                        dataType: {
                            type: utils_1.default.toTitleCase(table.name),
                            isArray: true,
                            relationType: '1-n',
                            isRelationHolder: true,
                        },
                    };
                    updatedSchema = _this.addColumnToTable(updatedSchema, column.dataType.references ? column.dataType.references.table : '', targetColumn);
                    updatedSchema = _this.addColumnToTable(updatedSchema, table.name, sourceColumn);
                }
            });
            return table;
        });
        return updatedSchema;
    };
    /**
     * Normalizes many to many relations.
     * Parses the table list and checks for tables
     * which are CrossReferenceTables(many to many)
     * and adds the foreign columns to the source and target tables.
     *
     * @param {*} schema db schema
     * @returns {*} normalized db schema
     */
    MysqlSchemaPreprocessor.prototype.normalizeManyToManyRelations = function (schema) {
        var _this = this;
        var updatedSchema = schema;
        schema.forEach(function (table) {
            if (!_this.tableIsCrossReferenceTable(table) || table.columns.length !== CROSS_REFERENCE_COL_LENGTH) {
                return table;
            }
            var source = table.columns[FIRST_INDEX];
            var target = table.columns[SECOND_INDEX];
            var sourceColumn = {
                name: source.dataType.references ? source.dataType.references.table : '',
                primary: source.primary,
                unique: false,
                allowNull: true,
                dataType: {
                    type: utils_1.default.toTitleCase(source.dataType.type),
                    isArray: true,
                    relationType: 'n-n',
                    isRelationHolder: true,
                },
            };
            var targetColumn = {
                name: target.dataType.references ? target.dataType.references.table : '',
                primary: target.primary,
                unique: false,
                allowNull: true,
                dataType: {
                    type: utils_1.default.toTitleCase(target.dataType.type),
                    isArray: true,
                    relationType: 'n-n',
                    isRelationHolder: true,
                },
            };
            updatedSchema = _this.addColumnToTable(updatedSchema, source.dataType.references ? source.dataType.references.table : '', targetColumn);
            updatedSchema = _this.addColumnToTable(updatedSchema, target.dataType.references ? target.dataType.references.table : '', sourceColumn);
            updatedSchema = _this.removeColumnFromTable(updatedSchema, table.name, source.name);
            updatedSchema = _this.removeColumnFromTable(updatedSchema, table.name, target.name);
            return table;
        });
        return updatedSchema;
    };
    /**
     * Strips tables with no columns from schema.
     *
     * @param {*} schema db schema
     * @returns {Array<*>} cleaned up schema
     */
    MysqlSchemaPreprocessor.prototype.stripEmptyTables = function (schema) {
        return schema.filter(function (table) { return Boolean(Object.keys(table.columns).length); });
    };
    /**
     * Checks if the table has any foreign key columns.
     *
     * @param {*} table db table
     * @returns {boolean} checked if it contains foreign keys.
     */
    MysqlSchemaPreprocessor.prototype.tableHasForeignKeys = function (table) {
        return Boolean(table.columns.filter(function (column) { return column.foreignKey; }).length);
    };
    /**
     * Checks if the table is a cross reference map(association table for many to many relations).
     *
     * @param {*} table table schema
     * @returns {boolean} checked if cross reference table.
     */
    MysqlSchemaPreprocessor.prototype.tableIsCrossReferenceTable = function (table) {
        return table.columns.filter(function (column) { return column.foreignKey; }).length === table.columns.length;
    };
    /**
     * Adds a column to a table in the schema.
     *
     * @param {*} schema db schema
     * @param {string} tableName table in which to add column
     * @param {*} column column to be added
     * @returns {Array|*} updated schema
     */
    MysqlSchemaPreprocessor.prototype.addColumnToTable = function (schema, tableName, column) {
        return this.removeColumnFromTable(schema, tableName, column.name).map(function (table) {
            if (table.name !== tableName) {
                return table;
            }
            table.columns.push(column);
            return table;
        });
    };
    /**
     * Removes a column from a table in the schema.
     *
     * @param {*} schema db schema
     * @param {string} tableName table from which we need to remove
     * @param {string} columnName column to be removed
     * @returns {Array|*} schema with removed column
     */
    MysqlSchemaPreprocessor.prototype.removeColumnFromTable = function (schema, tableName, columnName) {
        return schema.map(function (table) {
            if (table.name !== tableName) {
                return table;
            }
            table.columns = table.columns.filter(function (column) { return column.name !== columnName; });
            return table;
        });
    };
    /**
     * Remove unused properties from schema.
     *
     * @param {*} schema db schema
     * @returns {Array|*} cleaned up db schema
     */
    MysqlSchemaPreprocessor.prototype.cleanupUnusedPropertiesFromColumns = function (schema) {
        return schema.map(function (table) {
            table.columns.map(function (column) {
                delete column.foreignKey;
                return column;
            });
            return table;
        });
    };
    return MysqlSchemaPreprocessor;
}());
exports.default = MysqlSchemaPreprocessor;
