"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var typeUtil_1 = require("../commons/utils/typeUtil");
var schemaUtil_1 = require("../commons/utils/schemaUtil");
/**
 * Raml schema pre processor.
 *
 * @export
 * @class RamlSchemaPreprocessor
 */
var RamlSchemaPreprocessor = (function () {
    function RamlSchemaPreprocessor() {
    }
    /**
     * Normalize the table schema.
     *
     * @param {RamlColumnSchema} columnSchema column schema to be normalized
     * @returns {Column} normalized column schema
     */
    RamlSchemaPreprocessor.prototype.convertToStandardSchema = function (columnSchema) {
        return {
            name: columnSchema.name,
            primary: columnSchema.name === 'id',
            unique: columnSchema.name === 'id',
            allowNull: !columnSchema.required,
            dataType: typeUtil_1.default.convertRamlTypes({
                type: columnSchema.type.pop() || '',
                items: columnSchema.items,
            }),
        };
    };
    /**
     * Normalize schema relations.
     *
     * @param {Schema} schema - schema which needs to be normalized
     * @returns {Schema} normalized schema
     */
    RamlSchemaPreprocessor.prototype.normalizeSchemaRelations = function (schema) {
        var normalizedSchema = schema;
        normalizedSchema = this.normalizeOneToOneRelation(normalizedSchema);
        normalizedSchema = this.normalizeOneToManyRelation(normalizedSchema);
        normalizedSchema = this.normalizeManyToManyRelation(normalizedSchema);
        return normalizedSchema;
    };
    /**
     * Adds the relation type to schema.
     *
     * @param {Schema} schema - schema which needs normalization
     * @return {Schema} normalized schema
     */
    RamlSchemaPreprocessor.prototype.normalizeOneToOneRelation = function (schema) {
        var updatedSchema = schema;
        updatedSchema.map(function (table) {
            table.columns.map(function (column) {
                if (!column.allowNull &&
                    !column.dataType.isArray &&
                    schemaUtil_1.default.isCircularRelation(table, column, schema) &&
                    !schemaUtil_1.default.circularRelationIsRequired(table, column, schema) &&
                    !schemaUtil_1.default.circularRelationIsArray(table, column, schema) &&
                    !typeUtil_1.default.isDefaultType(column.dataType.type)) {
                    column.dataType.relationType = '1-1';
                }
                else if (column.allowNull &&
                    !column.dataType.isArray &&
                    schemaUtil_1.default.isCircularRelation(table, column, schema) &&
                    schemaUtil_1.default.circularRelationIsRequired(table, column, schema) &&
                    !schemaUtil_1.default.circularRelationIsArray(table, column, schema) &&
                    !typeUtil_1.default.isDefaultType(column.dataType.type)) {
                    column.dataType.relationType = '1-1';
                    column.dataType.isRelationHolder = true;
                }
                return column;
            });
            return table;
        });
        return updatedSchema;
    };
    /**
     * Adds the relation type to schema.
     *
     * @param {Schema} schema - schema which needs normalization
     * @return {Schema} normalized schema
     */
    RamlSchemaPreprocessor.prototype.normalizeOneToManyRelation = function (schema) {
        var updatedSchema = schema;
        updatedSchema.map(function (table) {
            table.columns.map(function (column) {
                if (column.allowNull &&
                    column.dataType.isArray &&
                    schemaUtil_1.default.isCircularRelation(table, column, schema) &&
                    schemaUtil_1.default.circularRelationIsRequired(table, column, schema) &&
                    !schemaUtil_1.default.circularRelationIsArray(table, column, schema) &&
                    !typeUtil_1.default.isDefaultType(column.dataType.type)) {
                    column.dataType.relationType = '1-n';
                    column.dataType.isRelationHolder = true;
                }
                else if (!column.allowNull &&
                    !column.dataType.isArray &&
                    schemaUtil_1.default.isCircularRelation(table, column, schema) &&
                    !schemaUtil_1.default.circularRelationIsRequired(table, column, schema) &&
                    schemaUtil_1.default.circularRelationIsArray(table, column, schema) &&
                    !typeUtil_1.default.isDefaultType(column.dataType.type)) {
                    column.dataType.relationType = '1-n';
                }
                return column;
            });
            return table;
        });
        return updatedSchema;
    };
    /**
     * Adds the relation type to schema.
     *
     * @param {Schema} schema - schema which needs normalization
     * @return {Schema} normalized schema
     */
    RamlSchemaPreprocessor.prototype.normalizeManyToManyRelation = function (schema) {
        var updatedSchema = schema;
        updatedSchema.map(function (table) {
            table.columns.map(function (column) {
                if (column.allowNull &&
                    column.dataType.isArray &&
                    schemaUtil_1.default.isCircularRelation(table, column, schema) &&
                    schemaUtil_1.default.circularRelationIsArray(table, column, schema) &&
                    !schemaUtil_1.default.circularRelationIsRequired(table, column, schema) &&
                    !typeUtil_1.default.isDefaultType(column.dataType.type)) {
                    column.dataType.relationType = 'n-n';
                    column.dataType.isRelationHolder = true;
                }
                return column;
            });
            return table;
        });
        return updatedSchema;
    };
    return RamlSchemaPreprocessor;
}());
exports.default = RamlSchemaPreprocessor;
